#####################################################################################################
#												    #
#	This archive is part of the free distribution of data and statistical software code for	    #
#	"Generalized Linear Models: A Unified Approach", Jeff Gill, Sage QASS Series.  You are	    #
#	free to use, modify, distribute, publish, etc. provided attribution.  Please forward 	    #
#	bugs, complaints, comments, and useful changes to: jgill@latte.harvard.edu.  		    #
#											    	    #
#####################################################################################################

###################################### SETUP ########################################################
copper.data <- as.matrix(read.table("copper.dat",header=T,row.names=1))
copper.factors <- data.frame(as.matrix(copper.data))
attach(copper.factors)
#####################################################################################################

###################################### LINEAR MODEL #################################################
copper.stage1.linear <- glm(COPPERPRICE ~ INCOMEINDEX + ALUMPRICE + INVENTORYINDEX + TIME,
	family=gaussian,data=copper.factors)
copper.stage2.linear <- glm(WORLDCONSUMPTION ~ 
	copper.stage1.linear$fitted.values + INCOMEINDEX + ALUMPRICE,
	family=gaussian)

summary(copper.stage2.linear)
copper.stage2.linear$fitted.values
min(abs( eigen(-glm.vc(copper.stage2))$values^(-1) ))

diag.plot(copper.stage2.linear)
#####################################################################################################

###################################### GAMMA MODEL ##################################################
PRIMERATE <- c(2.56,3,3.17,3.05,3.16,3.77,4.2,3.83,4.48,4.82,4.5,4.5,4.5,4.5,4.54, 5.63,5.61,6.3,
	7.96,7.91,5.72,5.25,8.03,10.81,7.86)
NETGOODS_US <- c(3067,2611,1437,2576,2897,4753,6271,3462,1148,4892,5571,4521,5224,6801,4951,3817,3800,
	635,607,2603,-2260,-6416,911,-5505,8903)
copper.stage1 <- glm(COPPERPRICE ~ INCOMEINDEX + ALUMPRICE + INVENTORYINDEX + log(TIME), 
	family=Gamma)
copper.stage2 <- glm(WORLDCONSUMPTION ~ 
	copper.stage1$fitted.values + INCOMEINDEX + ALUMPRICE,
	family=Gamma)

copper.vc <- solve(-glm.vc(copper.stage2))
normed.copper.vc <- (copper.vc - mean(copper.vc))/sqrt(var(copper.vc))
1/eigen(normed.copper.vc)$values
summary(copper.stage2)
#####################################################################################################

###################################### FIRST DIFFERENCES ############################################
copper.q1 <- c( 1, quantile(copper.stage1$fitted.values,.25),
		  apply( cbind( as.matrix(INCOMEINDEX), as.matrix(ALUMPRICE)),2,mean) )
copper.q3 <- c( 1, quantile(copper.stage1$fitted.values,.75),
		  apply( cbind( as.matrix(INCOMEINDEX), as.matrix(ALUMPRICE)),2,mean) )

fd1 <- (copper.stage2$coefficients %*% copper.q1)^{-1} 

fd3 <- (copper.stage2$coefficients %*% copper.q3)^{-1}

fd3-fd1 

#####################################################################################################

###################################### RESIDUALS ANALYSIS ###########################################
copper.pears <- resid(copper.stage2,type="pearson")
copper.resp <- resid(copper.stage2,type="response")
copper.work <- resid(copper.stage2,type="working")
copper.dev <- resid(copper.stage2,type="deviance")

par(mfrow=c(1,4))
hist(copper.pears)
hist(copper.resp)
hist(copper.work)
hist(copper.dev)
#####################################################################################################

###################################### FIGURE 1: MODEL FIT DIAGNOSTICS ##############################
postscript("glm.fig4.ps")
diag.plot(copper.stage2)
dev.off()

diag.plot <- function(in.model)  {
	par(mfrow=c(1,3),mar=c(8,8,6,2))

	plot(in.model$fitted.values,in.model$y,xlab="",ylab="")
	mtext(side=1,outer=F,"Fitted Values",line=6,cex=1.5)
	mtext(side=2,outer=F,"Observed Values",line=4,cex=1.3)
	mtext(side=3,outer=F,"Model Fit Plot",line=3,cex=1.4)
	abline(lm(in.model$y~in.model$fitted.values)$coefficients)

	plot(in.model$fitted.values,resid(in.model,type="pearson"),xlab="",ylab="")
	mtext(side=1,outer=F,"Fitted Values",line=6,cex=1.5)
	mtext(side=2,outer=F,"Pearson Residuals",line=4,cex=1.3)
	mtext(side=3,outer=F,"Residual Dependence Plot",line=3,cex=1.4)
	abline(0,0)

	qqnorm.jg(resid(in.model,type="deviance"),main="",xlab="",ylab="")
	mtext(side=1,outer=F,"Quantiles of N(0,1)",line=6,cex=1.5)
	mtext(side=2,outer=F,"Deviance Residual Quantiles",line=4,cex=1.3)
	mtext(side=3,outer=F,"Normal-Quantile Plot",line=3,cex=1.4)
}

qqnorm.jg <- function (y, ylim, main = "Normal Q-Q Plot", xlab = "Theoretical Quantiles", 
            ylab = "Sample Quantiles", plot.it = TRUE, ...) 
{
            y <- y[!is.na(y)]
            if (!length(y)) 
                    stop("y is empty")
            if (missing(ylim)) 
                    ylim <- range(y)
            x <- qnorm((1:length(y) - 0.5)/length(y))
            if (plot.it) 
                    plot(x, sort(y), main = main, xlab = xlab, ylab = ylab, 
                            ylim = ylim, ...)
            invisible(list(x = x, y = y))
	    abline( lm(sort(y)~x)$coefficients )
}
#####################################################################################################
